#include "cyrkbdset.h"
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include <qdir.h>
#include <qcombobox.h>
#include <qlistview.h>

#include <qpe/qpeapplication.h>
#include <qpe/resource.h>
#include <qpe/config.h>
#include <qpe/applnk.h>
#include <qpe/qcopenvelope_qws.h>
#include <qcopchannel_qws.h>


/* 
 *  Constructs a MobileMsg which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
CyrKbdSet::CyrKbdSet( QWidget* parent,  const char* name, WFlags fl )
    : CyrKbdSetBase( parent, name, fl )
{
    Config cfg("cyrkbd");
    cfg.setGroup("Keymap");
    QStringList layouts = cfg.readListEntry("Layouts", QChar(','));

    for ( QStringList::Iterator it = layouts.begin(); it != layouts.end(); ++it )
    {
    	QListViewItem* item = new QListViewItem(keymap_list, *it);
    	item->setPixmap(0, loadFlag(*it));
    	keymap_list->insertItem(item);  
    }

    QDir d(QDir::homeDirPath() + "/Applications/cyrillica");
    d.setFilter( QDir::Files );
    for (unsigned int i = 0; i < d.count(); i++) 
		keymap_box->insertItem(loadFlag(d[i]), d[i]);
	
    cfg.setGroup("Settings");
	adaptiveSwitch->setChecked(cfg.readBoolEntry("adaptiveSwitch", true));
	
}

/*  
 *  Destroys the object and frees any allocated resources
 */
CyrKbdSet::~CyrKbdSet()
{
    // no need to delete child widgets, Qt does it all for us
    QCopEnvelope env( "CyrInput/Private", "reload()");
}

/*
 *  A simple slot... not very interesting.
 */
void CyrKbdSet::goodBye()
{
    close();
}

void CyrKbdSet::accept()
{
    Config cfg("cyrkbd");
    cfg.setGroup("Keymap");

    QStringList layouts;

    QListViewItem *item = keymap_list->firstChild();
    while (item) 
    {
		layouts.append(item->text(0));
		item = item->nextSibling();
    }

    cfg.writeEntry("Layouts", layouts, QChar(','));

    cfg.setGroup("Settings");
	cfg.writeEntry("adaptiveSwitch", adaptiveSwitch->isChecked());

    QDialog::accept();
    close();

}

void CyrKbdSet::add_keymap()
{
    QListViewItem *item = keymap_list->firstChild();
    while (item) 
    {
		if (keymap_box->currentText() == item->text(0)) 
			return;

		item = item->nextSibling();
    }

	item = new QListViewItem(keymap_list, keymap_box->currentText());
	item->setPixmap(0, loadFlag(keymap_box->currentText()));
	keymap_list->insertItem(item);  
}

void CyrKbdSet::remove_keymap()
{
    QListViewItem *item = keymap_list->currentItem();
    if (!item) return;
    keymap_list->takeItem(item);
}

QPixmap CyrKbdSet::loadFlag(const QString& label)
{
    QPixmap flag;
    flag.convertFromImage(Resource::loadImage("cyrillica/" + label).smoothScale(AppLnk::smallIconSize(), AppLnk::smallIconSize()));
	return flag;    
}
