#include "keybshow.h"

#include <qmainwindow.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qpopupmenu.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpixmap.h>
#include <qiconset.h> 
#include <stdlib.h>

#include <qpe/global.h>
#include <qpe/qpeapplication.h>
#include <qpe/config.h>
#include <qpe/applnk.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/resource.h> 

#include <qcopchannel_qws.h>
#include "../config.h"

#include <stdio.h>

#define MAX_LAYOUTS 20
QString kMsg[MAX_LAYOUTS];
QString kName[MAX_LAYOUTS];

KeybShow::KeybShow( QWidget *parent ) : QLabel( parent )
{
    _chanel = new QCopChannel( "CyrInput/Private", this );
    connect( _chanel, SIGNAL( received(const QCString &, const QByteArray &)),
           this, SLOT(message( const QCString& )) );
		   
//	QPEApplication::setStylusOperation(this, QPEApplication::RightOnHold);
//	connect(this, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)),
//        this, SLOT( viewMount(QListViewItem *)));	

	m_pMenu = new QPopupMenu(this);
    m_pTimer = new QTimer(this);
	connect(m_pTimer, SIGNAL(timeout()), this, SLOT(displayLayouts()));

#if (SLC700 == 1)
    setFont( QFont( "Helvetica", 18, QFont::Normal ) );
#else
    setFont( QFont( "Helvetica", 10, QFont::Normal ) );
#endif

	defaultBackgroundColor = backgroundColor();
    load_config();
}

void KeybShow::mousePressEvent(QMouseEvent*)
{
	mButtonDown = true;
	m_pTimer->start(1000, true);
}

void KeybShow::mouseReleaseEvent(QMouseEvent*)
{
	mButtonDown = false;
	nextLayout();
}

void KeybShow::displayLayouts()
{
	if (mButtonDown && !m_pMenu->isVisible())
	{
		m_pMenu->clear();
		for (int i = 0; i < numLayouts; i++)
		{
			int item = m_pMenu->insertItem(QIconSet(loadFlag(kMsg[i])), kName[i] + (i == lang ? "*" : ""));
			if (i == lang)
				m_pMenu->setItemChecked(item, true);
		}
		
		connect(m_pMenu, SIGNAL(activated(int)), this, SLOT(menuItemClicked(int)));
		QPoint curPos = mapToGlobal(QPoint(0,0));
		QSize sh = m_pMenu->sizeHint();
		m_pMenu->popup( curPos-QPoint((sh.width()-width())/2, sh.height()));
	}
}

void KeybShow::menuItemClicked(int item)
{
	int idx = m_pMenu->indexOf(item);
	if (idx == -1 || idx == lang)
		return;

	prevLang = lang;
	lang = idx;

    setLabel(lang);
    QCopEnvelope env( "CyrInput/Private", "loadkmap(QString)");
    env << kMsg[lang];
}

void KeybShow::message( const QCString& message )
{
    //int i = 0;

    if (message == "reload()") 
	{
		load_config();
		return;
    }

    if (message == "nextKbdLayout()") 
	{
		nextLayout();
    }
}

void KeybShow::nextLayout()
{
	if (adaptiveSwitch && numLayouts > 2)
	{
		int currentLang = lang;
		
		if (prevLang != -1)
			lang = prevLang;
		else
			lang++;
			
		prevLang = currentLang;
	}
	else
   		lang++;
	
	
    if (kMsg[lang] == "") lang = 0;
    setLabel(lang);
    
    QCopEnvelope env( "CyrInput/Private", "loadkmap(QString)");
    env << kMsg[lang];
}

bool KeybShow::checkMap(QString fname, QString& n)
{
    QString filename = QDir::homeDirPath() + "/Applications/cyrillica/" + fname;
    QFile f(filename);
    if (!f.exists()) return false;
    if (f.open(IO_ReadOnly)) {
	QTextStream t(&f);
	QString s;
	while (!t.eof()) {
	    s = t.readLine();
	    if (s == "[label]") {
		n = t.readLine();
		f.close();
		return true;
	    }
	}
	f.close();
    }
    return false;
}

void KeybShow::load_config()
{
//    int f, i;

    Config cfg("cyrkbd");
    cfg.setGroup("Keymap");
    QStringList layouts = cfg.readListEntry("Layouts", QChar(','));

//    FILE *fd = fopen("/tmp/sux.txt","w");
    int i = 0;
	numLayouts = 0;
    for ( QStringList::Iterator it = layouts.begin(); it != layouts.end(); ++it ) 
	{
		if (i >= MAX_LAYOUTS - 1)
			break;
			
		if (checkMap(*it, kName[i])) 
		{
	    	kMsg[i] = *it;
//	    	fprintf(fd, "%s %s\n", kMsg[i].latin1(), kName[i].latin1());
	    	i++;
			numLayouts++;
		}
    }
//    fprintf(fd, "%d\n", i);
//    fclose(fd);

	prevLang = -1;

    if (i == 0) {
        QCopEnvelope env( "CyrInput/Private", "disable()");
    } else {
        kName[i] = "";
        kMsg[i] = "";

        QCopEnvelope env( "CyrInput/Private", "enable()");

        lang = 0;
        setLabel(lang);
        QCopEnvelope env1( "CyrInput/Private", "loadkmap(QString)");
        env1 << kMsg[lang];
    }
    
    if (i > 1) show();
    else hide();

    cfg.setGroup("Settings");
	adaptiveSwitch = cfg.readBoolEntry("adaptiveSwitch", 0);
}

QPixmap KeybShow::loadFlag(const QString& label)
{
    QPixmap flag;
    flag.convertFromImage(Resource::loadImage("cyrillica/" + label).smoothScale(AppLnk::smallIconSize(), AppLnk::smallIconSize()));
	return flag;    
}

void KeybShow::setLabel(int lang)
{
	QPixmap flag = loadFlag(kMsg[lang]);
    if (flag.isNull())
    {
	    setBackgroundColor(QColor(80, 255, 80));
	    setText(kName[lang]);
    }
	else
	{
		setBackgroundColor(defaultBackgroundColor);
		setPixmap(flag);
	}
}
